@@ -44,7 +44,7 @@
 #define _SYS_OPENBSD_
 #define TCSYSNAME   "OpenBSD"
 
-#elif defined(__sun__)
+#elif defined(__sun__) || defined(__sun)
 
 #define _SYS_SUNOS_
 #define TCSYSNAME   "SunOS"
@@ -259,6 +259,7 @@
 
 #if defined(_SYS_FREEBSD_) || defined(_SYS_NETBSD_) || defined(_SYS_OPENBSD_)
 #define nan(TC_a)      strtod("nan", NULL)
+#define nanl(TC_a)     ((long double)strtod("nan", NULL))
 #endif
 
 #if ! defined(PATH_MAX)
