@@ -31,6 +31,7 @@ static void printerr(TCADB *adb);
 static int sepstrtochr(const char *str);
 static char *strtozsv(const char *str, int sep, int *sp);
 static int printdata(const char *ptr, int size, bool px, int sep);
+static void setskeltran(ADBSKEL *skel);
 static bool mapbdbproc(void *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
                        void *op);
 static int runcreate(int argc, char **argv);
@@ -160,6 +161,41 @@ static int printdata(const char *ptr, int size, bool px, int sep){
 }
 
 
+/* set the transparent skeleton database */
+static void setskeltran(ADBSKEL *skel){
+  memset(skel, 0, sizeof(*skel));
+  skel->opq = tcadbnew();
+  skel->del = (void (*)(void *))tcadbdel;
+  skel->open = (bool (*)(void *, const char *))tcadbopen;
+  skel->close = (bool (*)(void *))tcadbclose;
+  skel->put = (bool (*)(void *, const void *, int, const void *, int))tcadbput;
+  skel->putkeep = (bool (*)(void *, const void *, int, const void *, int))tcadbputkeep;
+  skel->putcat = (bool (*)(void *, const void *, int, const void *, int))tcadbputcat;
+  skel->out = (bool (*)(void *, const void *, int))tcadbout;
+  skel->get = (void *(*)(void *, const void *, int, int *))tcadbget;
+  skel->vsiz = (int (*)(void *, const void *, int))tcadbvsiz;
+  skel->iterinit = (bool (*)(void *))tcadbiterinit;
+  skel->iternext = (void *(*)(void *, int *))tcadbiternext;
+  skel->fwmkeys = (TCLIST *(*)(void *, const void *, int, int))tcadbfwmkeys;
+  skel->addint = (int (*)(void *, const void *, int, int))tcadbaddint;
+  skel->adddouble = (double (*)(void *, const void *, int, double))tcadbadddouble;
+  skel->sync = (bool (*)(void *))tcadbsync;
+  skel->optimize = (bool (*)(void *, const char *))tcadboptimize;
+  skel->vanish = (bool (*)(void *))tcadbvanish;
+  skel->copy = (bool (*)(void *, const char *))tcadbcopy;
+  skel->tranbegin = (bool (*)(void *))tcadbtranbegin;
+  skel->trancommit = (bool (*)(void *))tcadbtrancommit;
+  skel->tranabort = (bool (*)(void *))tcadbtranabort;
+  skel->path = (const char *(*)(void *))tcadbpath;
+  skel->rnum = (uint64_t (*)(void *))tcadbrnum;
+  skel->size = (uint64_t (*)(void *))tcadbsize;
+  skel->misc = (TCLIST *(*)(void *, const char *, const TCLIST *))tcadbmisc;
+  skel->putproc =
+    (bool (*)(void *, const void *, int, const void *, int, TCPDPROC, void *))tcadbputproc;
+  skel->foreach = (bool (*)(void *, TCITER, void *))tcadbforeach;
+}
+
+
 /* mapping function */
 static bool mapbdbproc(void *map, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
                        void *op){
@@ -501,6 +537,24 @@ static int runversion(int argc, char **argv){
 /* perform create command */
 static int proccreate(const char *name){
   TCADB *adb = tcadbnew();
+  ADBSKEL skel;
+  if(*name == '@'){
+    setskeltran(&skel);
+    if(!tcadbsetskel(adb, &skel)){
+      printerr(adb);
+      skel.del(skel.opq);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  } else if(*name == '%'){
+    if(!tcadbsetskelmulti(adb, 8)){
+      printerr(adb);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  }
   if(!tcadbopen(adb, name)){
     printerr(adb);
     tcadbdel(adb);
@@ -519,6 +573,24 @@ static int proccreate(const char *name){
 /* perform inform command */
 static int procinform(const char *name){
   TCADB *adb = tcadbnew();
+  ADBSKEL skel;
+  if(*name == '@'){
+    setskeltran(&skel);
+    if(!tcadbsetskel(adb, &skel)){
+      printerr(adb);
+      skel.del(skel.opq);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  } else if(*name == '%'){
+    if(!tcadbsetskelmulti(adb, 8)){
+      printerr(adb);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  }
   if(!tcadbopen(adb, name)){
     printerr(adb);
     tcadbdel(adb);
@@ -537,6 +609,7 @@ static int procinform(const char *name){
   case ADBOBDB: type = "B+ tree database"; break;
   case ADBOFDB: type = "fixed-length database"; break;
   case ADBOTDB: type = "table database"; break;
+  case ADBOSKEL: type = "skeleton database"; break;
   }
   printf("database type: %s\n", type);
   printf("record number: %llu\n", (unsigned long long)tcadbrnum(adb));
@@ -554,6 +627,24 @@ static int procinform(const char *name){
 static int procput(const char *name, const char *kbuf, int ksiz, const char *vbuf, int vsiz,
                    int dmode){
   TCADB *adb = tcadbnew();
+  ADBSKEL skel;
+  if(*name == '@'){
+    setskeltran(&skel);
+    if(!tcadbsetskel(adb, &skel)){
+      printerr(adb);
+      skel.del(skel.opq);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  } else if(*name == '%'){
+    if(!tcadbsetskelmulti(adb, 8)){
+      printerr(adb);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  }
   if(!tcadbopen(adb, name)){
     printerr(adb);
     tcadbdel(adb);
@@ -604,6 +695,24 @@ static int procput(const char *name, const char *kbuf, int ksiz, const char *vbu
 /* perform out command */
 static int procout(const char *name, const char *kbuf, int ksiz){
   TCADB *adb = tcadbnew();
+  ADBSKEL skel;
+  if(*name == '@'){
+    setskeltran(&skel);
+    if(!tcadbsetskel(adb, &skel)){
+      printerr(adb);
+      skel.del(skel.opq);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  } else if(*name == '%'){
+    if(!tcadbsetskelmulti(adb, 8)){
+      printerr(adb);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  }
   if(!tcadbopen(adb, name)){
     printerr(adb);
     tcadbdel(adb);
@@ -626,6 +735,24 @@ static int procout(const char *name, const char *kbuf, int ksiz){
 /* perform get command */
 static int procget(const char *name, const char *kbuf, int ksiz, int sep, bool px, bool pz){
   TCADB *adb = tcadbnew();
+  ADBSKEL skel;
+  if(*name == '@'){
+    setskeltran(&skel);
+    if(!tcadbsetskel(adb, &skel)){
+      printerr(adb);
+      skel.del(skel.opq);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  } else if(*name == '%'){
+    if(!tcadbsetskelmulti(adb, 8)){
+      printerr(adb);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  }
   if(!tcadbopen(adb, name)){
     printerr(adb);
     tcadbdel(adb);
@@ -654,6 +781,24 @@ static int procget(const char *name, const char *kbuf, int ksiz, int sep, bool p
 /* perform list command */
 static int proclist(const char *name, int sep, int max, bool pv, bool px, const char *fmstr){
   TCADB *adb = tcadbnew();
+  ADBSKEL skel;
+  if(*name == '@'){
+    setskeltran(&skel);
+    if(!tcadbsetskel(adb, &skel)){
+      printerr(adb);
+      skel.del(skel.opq);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  } else if(*name == '%'){
+    if(!tcadbsetskelmulti(adb, 8)){
+      printerr(adb);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  }
   if(!tcadbopen(adb, name)){
     printerr(adb);
     tcadbdel(adb);
@@ -714,6 +859,24 @@ static int proclist(const char *name, int sep, int max, bool pv, bool px, const
 /* perform optimize command */
 static int procoptimize(const char *name, const char *params){
   TCADB *adb = tcadbnew();
+  ADBSKEL skel;
+  if(*name == '@'){
+    setskeltran(&skel);
+    if(!tcadbsetskel(adb, &skel)){
+      printerr(adb);
+      skel.del(skel.opq);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  } else if(*name == '%'){
+    if(!tcadbsetskelmulti(adb, 8)){
+      printerr(adb);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  }
   if(!tcadbopen(adb, name)){
     printerr(adb);
     tcadbdel(adb);
@@ -736,6 +899,24 @@ static int procoptimize(const char *name, const char *params){
 /* perform misc command */
 static int procmisc(const char *name, const char *func, const TCLIST *args, int sep, bool px){
   TCADB *adb = tcadbnew();
+  ADBSKEL skel;
+  if(*name == '@'){
+    setskeltran(&skel);
+    if(!tcadbsetskel(adb, &skel)){
+      printerr(adb);
+      skel.del(skel.opq);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  } else if(*name == '%'){
+    if(!tcadbsetskelmulti(adb, 8)){
+      printerr(adb);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  }
   if(!tcadbopen(adb, name)){
     printerr(adb);
     tcadbdel(adb);
@@ -767,6 +948,24 @@ static int procmisc(const char *name, const char *func, const TCLIST *args, int
 /* perform map command */
 static int procmap(const char *name, const char *dest, const char *fmstr){
   TCADB *adb = tcadbnew();
+  ADBSKEL skel;
+  if(*name == '@'){
+    setskeltran(&skel);
+    if(!tcadbsetskel(adb, &skel)){
+      printerr(adb);
+      skel.del(skel.opq);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  } else if(*name == '%'){
+    if(!tcadbsetskelmulti(adb, 8)){
+      printerr(adb);
+      tcadbdel(adb);
+      return 1;
+    }
+    name++;
+  }
   if(!tcadbopen(adb, name)){
     printerr(adb);
     tcadbdel(adb);
