@@ -131,6 +131,7 @@ int main(int argc, char **argv){
     }
   }
   if(tcadbsize(db) > 0){
+    if(wmode) tcadbtranbegin(db);
     switch(params.action){
     case ACTLIST:
     case ACTLISTVAL:
@@ -145,6 +146,7 @@ int main(int argc, char **argv){
       doerror(400, "no such action");
       break;
     }
+    if(wmode) tcadbtrancommit(db);
   } else {
     doerror(500, "the database file could not be opened");
   }
@@ -172,67 +174,7 @@ static void readparameters(TCMAP *params){
     buf = tcstrdup(rp);
     len = strlen(buf);
   }
-  if(buf && len > 0){
-    if((rp = getenv("CONTENT_TYPE")) != NULL && tcstrfwm(rp, "multipart/form-data") &&
-       (rp = strstr(rp, "boundary=")) != NULL){
-      rp += 9;
-      if(*rp == '"') rp++;
-      char bstr[strlen(rp)+1];
-      strcpy(bstr, rp);
-      char *wp = strchr(bstr, ';');
-      if(wp) *wp = '\0';
-      wp = strchr(bstr, '"');
-      if(wp) *wp = '\0';
-      TCLIST *parts = tcmimeparts(buf, len, bstr);
-      int pnum = tclistnum(parts);
-      for(int i = 0; i < pnum; i++){
-        int psiz;
-        const char *part = tclistval(parts, i, &psiz);
-        TCMAP *hmap = tcmapnew2(MINIBNUM);
-        int bsiz;
-        char *body = tcmimebreak(part, psiz, hmap, &bsiz);
-        int nsiz;
-        const char *name = tcmapget(hmap, "NAME", 4, &nsiz);
-        if(name){
-          tcmapput(params, name, nsiz, body, bsiz);
-          const char *fname = tcmapget2(hmap, "FILENAME");
-          if(fname){
-            if(*fname == '/'){
-              fname = strrchr(fname, '/') + 1;
-            } else if(((*fname >= 'a' && *fname <= 'z') || (*fname >= 'A' && *fname <= 'Z')) &&
-                      fname[1] == ':' && fname[2] == '\\'){
-              fname = strrchr(fname, '\\') + 1;
-            }
-            if(*fname != '\0'){
-              char key[nsiz+10];
-              sprintf(key, "%s_filename", name);
-              tcmapput2(params, key, fname);
-            }
-          }
-        }
-        tcfree(body);
-        tcmapdel(hmap);
-      }
-      tclistdel(parts);
-    } else {
-      TCLIST *pairs = tcstrsplit(buf, "&;");
-      int num = tclistnum(pairs);
-      for(int i = 0; i < num; i++){
-        char *key = tcstrdup(tclistval2(pairs, i));
-        char *val = strchr(key, '=');
-        if(val){
-          *(val++) = '\0';
-          char *dkey = tcurldecode(key, &len);
-          char *dval = tcurldecode(val, &len);
-          tcmapput2(params, dkey, dval);
-          tcfree(dval);
-          tcfree(dkey);
-        }
-        tcfree(key);
-      }
-      tclistdel(pairs);
-    }
-  }
+  if(buf && len > 0) tcwwwformdecode2(buf, len, getenv("CONTENT_TYPE"), params);
   tcfree(buf);
 }
 
@@ -413,8 +355,7 @@ static void sethtmlheader(PARAMS *params, TCXSTR *obuf, TCADB *db){
   XP("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />\n");
   XP("<meta http-equiv=\"Content-Style-Type\" content=\"text/css\" />\n");
   XP("<title>%@</title>\n", PAGETITLE);
-  XP("<style type=\"text/css\">\n");
-  XP("html { margin: 0em; padding: 0em; }\n");
+  XP("<style type=\"text/css\">html { margin: 0em; padding: 0em; }\n");
   XP("body { margin :0em; padding: 0.5em 1em; background: #eeeeee; color: #111111; }\n");
   XP("h1 { margin: 3px; padding: 0px; font-size: 125%%; }\n");
   XP("h1 a { color: #000000; }\n");
