@@ -27,12 +27,6 @@
 __TCFDB_CLINKAGEBEGIN
 
 
-#include <stdlib.h>
-#include <stdbool.h>
-#include <stdint.h>
-#include <time.h>
-#include <limits.h>
-#include <math.h>
 #include <tcutil.h>
 
 
@@ -49,6 +43,7 @@ typedef struct {                         /* type of structure for a fixed-length
   void *tmtx;                            /* mutex for transaction */
   void *wmtx;                            /* mutex for write ahead logging */
   void *eckey;                           /* key for thread specific error code */
+  char *rpath;                           /* real path for locking */
   uint8_t type;                          /* database type */
   uint8_t flags;                         /* additional flags */
   uint32_t width;                        /* width of the value of each record */
@@ -139,7 +134,7 @@ int tcfdbecode(TCFDB *fdb);
    `fdb' specifies the fixed-length database object which is not opened.
    If successful, the return value is true, else, it is false.
    Note that the mutual exclusion control is needed if the object is shared by plural threads and
-   this function should should be called before the database is opened. */
+   this function should be called before the database is opened. */
 bool tcfdbsetmutex(TCFDB *fdb);
 
 
@@ -484,7 +479,7 @@ char *tcfdbiternext3(TCFDB *fdb);
    `np' specifies the pointer to the variable into which the number of elements of the return
    value is assigned.
    If successful, the return value is the pointer to an array of ID numbers of the corresponding
-   records.  `NULL' is returned on failure.  This function does never fail and return an empty
+   records.  `NULL' is returned on failure.  This function does never fail.  It returns an empty
    array even if no key corresponds.
    Because the region of the return value is allocated with the `malloc' call, it should be
    released with the `free' call when it is no longer in use. */
@@ -502,7 +497,7 @@ uint64_t *tcfdbrange(TCFDB *fdb, int64_t lower, int64_t upper, int max, int *np)
    `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
    specified.
    The return value is a list object of the corresponding decimal keys.  This function does never
-   fail and return an empty list even if no key corresponds.
+   fail.  It returns an empty list even if no key corresponds.
    Because the object of the return value is created with the function `tclistnew', it should be
    deleted with the function `tclistdel' when it is no longer in use.  Note that this function
    may be very slow because every key in the database is scanned. */
@@ -518,7 +513,7 @@ TCLIST *tcfdbrange2(TCFDB *fdb, const void *lbuf, int lsiz, const void *ubuf, in
    `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
    specified.
    The return value is a list object of the corresponding decimal keys.  This function does never
-   fail and return an empty list even if no key corresponds.
+   fail.  It returns an empty list even if no key corresponds.
    Because the object of the return value is created with the function `tclistnew', it should be
    deleted with the function `tclistdel' when it is no longer in use.  Note that this function
    may be very slow because every key in the database is scanned. */
@@ -532,7 +527,7 @@ TCLIST *tcfdbrange3(TCFDB *fdb, const char *lstr, const char *ustr, int max);
    `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
    specified.
    The return value is a list object of the corresponding decimal keys.  This function does never
-   fail and return an empty list even if no key corresponds.
+   fail.  It returns an empty list even if no key corresponds.
    Because the object of the return value is created with the function `tclistnew', it should be
    deleted with the function `tclistdel' when it is no longer in use.  Note that this function
    may be very slow because every key in the database is scanned. */
@@ -545,7 +540,7 @@ TCLIST *tcfdbrange4(TCFDB *fdb, const void *ibuf, int isiz, int max);
    `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
    specified.
    The return value is a list object of the corresponding decimal keys.  This function does never
-   fail and return an empty list even if no key corresponds.
+   fail.  It returns an empty list even if no key corresponds.
    Because the object of the return value is created with the function `tclistnew', it should be
    deleted with the function `tclistdel' when it is no longer in use.  Note that this function
    may be very slow because every key in the database is scanned. */
