@@ -27,12 +27,6 @@
 __TCHDB_CLINKAGEBEGIN
 
 
-#include <stdlib.h>
-#include <stdbool.h>
-#include <stdint.h>
-#include <time.h>
-#include <limits.h>
-#include <math.h>
 #include <tcutil.h>
 
 
@@ -49,6 +43,7 @@ typedef struct {                         /* type of structure for a hash databas
   void *tmtx;                            /* mutex for transaction */
   void *wmtx;                            /* mutex for write ahead logging */
   void *eckey;                           /* key for thread specific error code */
+  char *rpath;                           /* real path for locking */
   uint8_t type;                          /* database type */
   uint8_t flags;                         /* additional flags */
   uint64_t bnum;                         /* number of the bucket array */
@@ -176,7 +171,7 @@ int tchdbecode(TCHDB *hdb);
    `hdb' specifies the hash database object which is not opened.
    If successful, the return value is true, else, it is false.
    Note that the mutual exclusion control is needed if the object is shared by plural threads and
-   this function should should be called before the database is opened. */
+   this function should be called before the database is opened. */
 bool tchdbsetmutex(TCHDB *hdb);
 
 
@@ -452,8 +447,8 @@ bool tchdbiternext3(TCHDB *hdb, TCXSTR *kxstr, TCXSTR *vxstr);
    `psiz' specifies the size of the region of the prefix.
    `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
    specified.
-   The return value is a list object of the corresponding keys.  This function does never fail
-   and return an empty list even if no key corresponds.
+   The return value is a list object of the corresponding keys.  This function does never fail.
+   It returns an empty list even if no key corresponds.
    Because the object of the return value is created with the function `tclistnew', it should be
    deleted with the function `tclistdel' when it is no longer in use.  Note that this function
    may be very slow because every key in the database is scanned. */
@@ -465,8 +460,8 @@ TCLIST *tchdbfwmkeys(TCHDB *hdb, const void *pbuf, int psiz, int max);
    `pstr' specifies the string of the prefix.
    `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
    specified.
-   The return value is a list object of the corresponding keys.  This function does never fail
-   and return an empty list even if no key corresponds.
+   The return value is a list object of the corresponding keys.  This function does never fail.
+   It returns an empty list even if no key corresponds.
    Because the object of the return value is created with the function `tclistnew', it should be
    deleted with the function `tclistdel' when it is no longer in use.  Note that this function
    may be very slow because every key in the database is scanned. */
@@ -632,12 +627,6 @@ bool tchdbhasmutex(TCHDB *hdb);
 bool tchdbmemsync(TCHDB *hdb, bool phys);
 
 
-/* Clear the cache of a hash tree database object.
-   `hdb' specifies the hash tree database object.
-   If successful, the return value is true, else, it is false. */
-bool tchdbcacheclear(TCHDB *hdb);
-
-
 /* Get the number of elements of the bucket array of a hash database object.
    `hdb' specifies the hash database object.
    The return value is the number of elements of the bucket array or 0 if the object does not
@@ -763,6 +752,12 @@ uint32_t tchdbdfunit(TCHDB *hdb);
 bool tchdbdefrag(TCHDB *hdb, int64_t step);
 
 
+/* Clear the cache of a hash tree database object.
+   `hdb' specifies the hash tree database object.
+   If successful, the return value is true, else, it is false. */
+bool tchdbcacheclear(TCHDB *hdb);
+
+
 /* Store a record into a hash database object with a duplication handler.
    `hdb' specifies the hash database object connected as a writer.
    `kbuf' specifies the pointer to the region of the key.
