@@ -27,12 +27,6 @@
 __TCTDB_CLINKAGEBEGIN
 
 
-#include <stdlib.h>
-#include <stdbool.h>
-#include <stdint.h>
-#include <time.h>
-#include <limits.h>
-#include <math.h>
 #include <tcutil.h>
 #include <tchdb.h>
 #include <tcbdb.h>
@@ -48,6 +42,7 @@ typedef struct {                         /* type of structure for a column index
   char *name;                            /* column name */
   int type;                              /* data type */
   void *db;                              /* internal database object */
+  void *cc;                              /* internal cache object */
 } TDBIDX;
 
 typedef struct {                         /* type of structure for a table database */
@@ -58,6 +53,8 @@ typedef struct {                         /* type of structure for a table databa
   uint8_t opts;                          /* options */
   int32_t lcnum;                         /* max number of cached leaves */
   int32_t ncnum;                         /* max number of cached nodes */
+  int64_t iccmax;                        /* maximum size of the inverted cache */
+  double iccsync;                        /* synchronization ratio of the inverted cache */
   TDBIDX *idxs;                          /* column indices */
   int inum;                              /* number of column indices */
   bool tran;                             /* whether in the transaction */
@@ -89,6 +86,8 @@ enum {                                   /* enumeration for open modes */
 enum {                                   /* enumeration for index types */
   TDBITLEXICAL,                          /* lexical string */
   TDBITDECIMAL,                          /* decimal string */
+  TDBITTOKEN,                            /* token inverted index */
+  TDBITQGRAM,                            /* q-gram inverted index */
   TDBITOPT = 9998,                       /* optimize */
   TDBITVOID = 9999,                      /* void */
   TDBITKEEP = 1 << 24                    /* keep existing index */
@@ -102,6 +101,9 @@ typedef struct {                         /* type of structure for a condition */
   bool noidx;                            /* no index flag */
   char *expr;                            /* operand expression */
   int esiz;                              /* size of the operand expression */
+  void *regex;                           /* regular expression object */
+  void *ftsunits;                        /* full-text search units */
+  int ftsnum;                            /* number of full-text search units */
   bool alive;                            /* alive flag */
 } TDBCOND;
 
@@ -133,6 +135,10 @@ enum {                                   /* enumeration for query conditions */
   TDBQCNUMLE,                            /* number is less than or equal to */
   TDBQCNUMBT,                            /* number is between two tokens of */
   TDBQCNUMOREQ,                          /* number is equal to at least one token in */
+  TDBQCFTSPH,                            /* full-text search with the phrase of */
+  TDBQCFTSAND,                           /* full-text search with all tokens in */
+  TDBQCFTSOR,                            /* full-text search with at least one token in */
+  TDBQCFTSEX,                            /* full-text search with the compound expression of */
   TDBQCNEGATE = 1 << 24,                 /* negation flag */
   TDBQCNOIDX = 1 << 25                   /* no index flag */
 };
@@ -144,6 +150,12 @@ enum {                                   /* enumeration for order types */
   TDBQONUMDESC                           /* number descending */
 };
 
+enum {                                   /* enumeration for set operation types */
+  TDBMSUNION,                            /* union */
+  TDBMSISECT,                            /* intersection */
+  TDBMSDIFF                              /* difference */
+};
+
 enum {                                   /* enumeration for post treatments */
   TDBQPPUT = 1 << 0,                     /* modify the record */
   TDBQPOUT = 1 << 1,                     /* remove the record */
@@ -197,7 +209,7 @@ int tctdbecode(TCTDB *tdb);
    `tdb' specifies the table database object which is not opened.
    If successful, the return value is true, else, it is false.
    Note that the mutual exclusion control is needed if the object is shared by plural threads and
-   this function should should be called before the database is opened. */
+   this function should be called before the database is opened. */
 bool tctdbsetmutex(TCTDB *tdb);
 
 
@@ -475,14 +487,28 @@ void *tctdbiternext(TCTDB *tdb, int *sp);
 char *tctdbiternext2(TCTDB *tdb);
 
 
+/* Get the columns of the next record of the iterator of a table database object.
+   `tdb' specifies the table database object.
+   If successful, the return value is a map object of the columns of the next record, else, it is
+   `NULL'.  `NULL' is returned when no record is to be get out of the iterator.  The primary key
+   is added into the map as a column of an empty string key.
+   Because the object of the return value is created with the function `tcmapnew', it should be
+   deleted with the function `tcmapdel' when it is no longer in use.  It is possible to access
+   every record by iteration of calling this function.  However, it is not assured if updating
+   the database is occurred while the iteration.  Besides, the order of this traversal access
+   method is arbitrary, so it is not assured that the order of storing matches the one of the
+   traversal access. */
+TCMAP *tctdbiternext3(TCTDB *tdb);
+
+
 /* Get forward matching primary keys in a table database object.
    `tdb' specifies the table database object.
    `pbuf' specifies the pointer to the region of the prefix.
    `psiz' specifies the size of the region of the prefix.
    `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
    specified.
-   The return value is a list object of the corresponding keys.  This function does never fail
-   and return an empty list even if no key corresponds.
+   The return value is a list object of the corresponding keys.  This function does never fail.
+   It returns an empty list even if no key corresponds.
    Because the object of the return value is created with the function `tclistnew', it should be
    deleted with the function `tclistdel' when it is no longer in use.  Note that this function
    may be very slow because every key in the database is scanned. */
@@ -494,8 +520,8 @@ TCLIST *tctdbfwmkeys(TCTDB *tdb, const void *pbuf, int psiz, int max);
    `pstr' specifies the string of the prefix.
    `max' specifies the maximum number of keys to be fetched.  If it is negative, no limit is
    specified.
-   The return value is a list object of the corresponding keys.  This function does never fail
-   and return an empty list even if no key corresponds.
+   The return value is a list object of the corresponding keys.  This function does never fail.
+   It returns an empty list even if no key corresponds.
    Because the object of the return value is created with the function `tclistnew', it should be
    deleted with the function `tclistdel' when it is no longer in use.  Note that this function
    may be very slow because every key in the database is scanned. */
@@ -621,9 +647,9 @@ uint64_t tctdbfsiz(TCTDB *tdb);
    `name' specifies the name of a column.  If the name of an existing index is specified, the
    index is rebuilt.  An empty string means the primary key.
    `type' specifies the index type: `TDBITLEXICAL' for lexical string, `TDBITDECIMAL' for decimal
-   string.  If it is `TDBITOPT', the index is optimized.  If it is `TDBITVOID', the index is
-   removed.  If `TDBITKEEP' is added by bitwise-or and the index exists, this function merely
-   returns failure.
+   string, `TDBITTOKEN' for token inverted index, `TDBITQGRAM' for q-gram inverted index.  If it
+   is `TDBITOPT', the index is optimized.  If it is `TDBITVOID', the index is removed.  If
+   `TDBITKEEP' is added by bitwise-or and the index exists, this function merely returns failure.
    If successful, the return value is true, else, it is false.
    Note that the setting indices should be set after the database is opened. */
 bool tctdbsetindex(TCTDB *tdb, const char *name, int type);
@@ -660,7 +686,10 @@ void tctdbqrydel(TDBQRY *qry);
    greater than or equal to the expression, `TDBQCNUMLT' for number which is less than the
    expression, `TDBQCNUMLE' for number which is less than or equal to the expression, `TDBQCNUMBT'
    for number which is between two tokens of the expression, `TDBQCNUMOREQ' for number which is
-   equal to at least one token in the expression.  All operations can be flagged by bitwise-or:
+   equal to at least one token in the expression, `TDBQCFTSPH' for full-text search with the
+   phrase of the expression, `TDBQCFTSAND' for full-text search with all tokens in the expression,
+   `TDBQCFTSOR' for full-text search with at least one token in the expression, `TDBQCFTSEX' for
+   full-text search with the compound expression.  All operations can be flagged by bitwise-or:
    `TDBQCNEGATE' for negation, `TDBQCNOIDX' for using no index.
    `expr' specifies an operand exression. */
 void tctdbqryaddcond(TDBQRY *qry, const char *name, int op, const char *expr);
@@ -686,7 +715,7 @@ void tctdbqrysetlimit(TDBQRY *qry, int max, int skip);
 /* Execute the search of a query object.
    `qry' specifies the query object.
    The return value is a list object of the primary keys of the corresponding records.  This
-   function does never fail and return an empty list even if no record corresponds.
+   function does never fail.  It returns an empty list even if no record corresponds.
    Because the object of the return value is created with the function `tclistnew', it should
    be deleted with the function `tclistdel' when it is no longer in use. */
 TCLIST *tctdbqrysearch(TDBQRY *qry);
@@ -712,7 +741,7 @@ bool tctdbqrysearchout(TDBQRY *qry);
 bool tctdbqryproc(TDBQRY *qry, TDBQRYPROC proc, void *op);
 
 
-/* Get the hint of a query object.
+/* Get the hint string of a query object.
    `qry' specifies the query object.
    The return value is the hint string.
    This function should be called after the query execution by `tctdbqrysearch' and so on.  The
@@ -720,6 +749,19 @@ bool tctdbqryproc(TDBQRY *qry, TDBQRYPROC proc, void *op);
 const char *tctdbqryhint(TDBQRY *qry);
 
 
+/* Retrieve records with multiple query objects and get the set of the result.
+   `qrys' specifies an array of the query objects.
+   `num' specifies the number of elements of the array.
+   `type' specifies a set operation type: `TDBMSUNION' for the union set, `TDBMSISECT' for the
+   intersection set, `TDBMSDIFF' for the difference set.
+   The return value is a list object of the primary keys of the corresponding records.  This
+   function does never fail.  It returns an empty list even if no record corresponds.
+   If the first query object has the order setting, the result array is sorted by the order.
+   Because the object of the return value is created with the function `tclistnew', it should be
+   deleted with the function `tclistdel' when it is no longer in use. */
+TCLIST *tctdbmetasearch(TDBQRY **qrys, int num, int type);
+
+
 
 /*************************************************************************************************
  * features for experts
@@ -839,6 +881,17 @@ int64_t tctdbuidseed(TCTDB *tdb);
 bool tctdbsetuidseed(TCTDB *tdb, int64_t seed);
 
 
+/* Set the parameters of the inverted cache of a table database object.
+   `tdb' specifies the table database object.
+   `iccmax' specifies the maximum size.  If it is not more than 0, the default value is
+   specified.  The default value is 67108864.
+   `iccsync' specifies synchronization ratio.  If it is not more than 0, the default value is
+   specified.  The default value is 0.01.
+   If successful, the return value is true, else, it is false.
+   Note that the caching parameters should be set before the database is opened. */
+bool tctdbsetinvcache(TCTDB *tdb, int64_t iccmax, double iccsync);
+
+
 /* Set the custom codec functions of a table database object.
    `tdb' specifies the table database object.
    `enc' specifies the pointer to the custom encoding function.  It receives four parameters.
@@ -872,6 +925,12 @@ uint32_t tctdbdfunit(TCTDB *tdb);
 bool tctdbdefrag(TCTDB *tdb, int64_t step);
 
 
+/* Clear the cache of a table tree database object.
+   `tdb' specifies the table tree database object.
+   If successful, the return value is true, else, it is false. */
+bool tctdbcacheclear(TCTDB *tdb);
+
+
 /* Store a record into a table database object with a duplication handler.
    `tdb' specifies the table database object connected as a writer.
    `pkbuf' specifies the pointer to the region of the primary key.
@@ -896,6 +955,23 @@ bool tctdbputproc(TCTDB *tdb, const void *pkbuf, int pksiz, const void *cbuf, in
                   TCPDPROC proc, void *op);
 
 
+/* Retrieve the value of a column of a record in a table database object.
+   `tdb' specifies the table database object.
+   `pkbuf' specifies the pointer to the region of the primary key.
+   `pksiz' specifies the size of the region of the primary key.
+   `nbuf' specifies the pointer to the region of the column name.
+   `nsiz' specifies the size of the region of the column name.
+   `sp' specifies the pointer to the variable into which the size of the region of the return
+   value is assigned.
+   If successful, the return value is the pointer to the region of the value of the column of the
+   corresponding record.  `NULL' is returned if no record corresponds or there is no column.
+   Because an additional zero code is appended at the end of the region of the return value,
+   the return value can be treated as a character string.  Because the region of the return
+   value is allocated with the `malloc' call, it should be released with the `free' call when
+   it is no longer in use. */
+char *tctdbget4(TCTDB *tdb, const void *pkbuf, int pksiz, const void *nbuf, int nsiz, int *sp);
+
+
 /* Move the iterator to the record corresponding a key of a table database object.
    `tdb' specifies the table database object.
    `pkbuf' specifies the pointer to the region of the primary key.
@@ -955,12 +1031,35 @@ bool tctdbqrysearchout2(TDBQRY *qry);
 int tctdbstrtoindextype(const char *str);
 
 
+/* Convert a string into the meta search type number.
+   `str' specifies a string.
+   The return value is the meta search type number or -1 on failure. */
+int tctdbstrtometasearcytype(const char *str);
+
+
 /* Get the count of corresponding records of a query object.
    `qry' specifies the query object.
    The return value is the count of corresponding records. */
 int tctdbqrycount(TDBQRY *qry);
 
 
+/* Generate keyword-in-context strings from a query object.
+   `qry' specifies the query object.
+   `cols' specifies a map object containing columns.
+   `name' specifies the name of a column.  If it is `NULL', the first column of the query is
+   specified.
+   `width' specifies the width of strings picked up around each keyword.
+   `opts' specifies options by bitwise-or: `TCKWMUTAB' specifies that each keyword is marked up
+   between two tab characters, `TCKWMUCTRL' specifies that each keyword is marked up by the STX
+   (0x02) code and the ETX (0x03) code, `TCKWMUBRCT' specifies that each keyword is marked up by
+   the two square brackets, `TCKWNOOVER' specifies that each context does not overlap,
+   `TCKWPULEAD' specifies that the lead string is picked up forcibly.
+   The return value is the list object whose elements are strings around keywords.
+   Because the object of the return value is created with the function `tclistnew', it should
+   be deleted with the function `tclistdel' when it is no longer in use. */
+TCLIST *tctdbqrykwic(TDBQRY *qry, TCMAP *cols, const char *name, int width, int opts);
+
+
 /* Convert a string into the query operation number.
    `str' specifies a string.
    The return value is the query operation number or -1 on failure. */
@@ -973,6 +1072,12 @@ int tctdbqrystrtocondop(const char *str);
 int tctdbqrystrtoordertype(const char *str);
 
 
+/* Convert a string into the set operation type number.
+   `str' specifies a string.
+   The return value is the set operation type or -1 on failure. */
+int tctdbmetastrtosettype(const char *str);
+
+
 
 __TCTDB_CLINKAGEEND
 #endif                                   /* duplication check */
