@@ -36,9 +36,12 @@ static int runtcbs(int argc, char **argv);
 static int runzlib(int argc, char **argv);
 static int runbzip(int argc, char **argv);
 static int runxml(int argc, char **argv);
+static int runcstr(int argc, char **argv);
 static int runucs(int argc, char **argv);
 static int runhash(int argc, char **argv);
+static int runcipher(int argc, char **argv);
 static int rundate(int argc, char **argv);
+static int runtmpl(int argc, char **argv);
 static int runconf(int argc, char **argv);
 static int procurl(const char *ibuf, int isiz, bool dec, bool br, const char *base);
 static int procbase(const char *ibuf, int isiz, bool dec);
@@ -51,9 +54,12 @@ static int proctcbs(const char *ibuf, int isiz, bool dec);
 static int proczlib(const char *ibuf, int isiz, bool dec, bool gz);
 static int procbzip(const char *ibuf, int isiz, bool dec);
 static int procxml(const char *ibuf, int isiz, bool dec, bool br);
-static int procucs(const char *ibuf, int isiz, bool dec);
+static int proccstr(const char *ibuf, int isiz, bool dec, bool js);
+static int procucs(const char *ibuf, int isiz, bool dec, bool un, const char *kw);
 static int prochash(const char *ibuf, int isiz, bool crc, int ch);
+static int proccipher(const char *ibuf, int isiz, const char *key);
 static int procdate(const char *str, int jl, bool wf, bool rf);
+static int proctmpl(const char *ibuf, int isiz, TCMAP *vars);
 static int procconf(int mode);
 
 
@@ -82,12 +88,18 @@ int main(int argc, char **argv){
     rv = runbzip(argc, argv);
   } else if(!strcmp(argv[1], "xml")){
     rv = runxml(argc, argv);
+  } else if(!strcmp(argv[1], "cstr")){
+    rv = runcstr(argc, argv);
   } else if(!strcmp(argv[1], "ucs")){
     rv = runucs(argc, argv);
   } else if(!strcmp(argv[1], "hash")){
     rv = runhash(argc, argv);
+  } else if(!strcmp(argv[1], "cipher")){
+    rv = runcipher(argc, argv);
   } else if(!strcmp(argv[1], "date")){
     rv = rundate(argc, argv);
+  } else if(!strcmp(argv[1], "tmpl")){
+    rv = runtmpl(argc, argv);
   } else if(!strcmp(argv[1], "conf")){
     rv = runconf(argc, argv);
   } else {
@@ -113,9 +125,12 @@ static void usage(void){
   fprintf(stderr, "  %s zlib [-d] [-gz] [file]\n", g_progname);
   fprintf(stderr, "  %s bzip [-d] [file]\n", g_progname);
   fprintf(stderr, "  %s xml [-d] [-br] [file]\n", g_progname);
-  fprintf(stderr, "  %s ucs [-d] [file]\n", g_progname);
+  fprintf(stderr, "  %s cstr [-d] [-js] [file]\n", g_progname);
+  fprintf(stderr, "  %s ucs [-d] [-un] [file]\n", g_progname);
   fprintf(stderr, "  %s hash [-crc] [-ch num] [file]\n", g_progname);
+  fprintf(stderr, "  %s cipher [-key str] [file]\n", g_progname);
   fprintf(stderr, "  %s date [-ds str] [-jl num] [-wf] [-rf]\n", g_progname);
+  fprintf(stderr, "  %s tmpl [-var name val] [file]\n", g_progname);
   fprintf(stderr, "  %s conf [-v|-i|-l|-p]\n", g_progname);
   fprintf(stderr, "\n");
   exit(1);
@@ -530,14 +545,17 @@ static int runxml(int argc, char **argv){
 }
 
 
-/* parse arguments of ucs command */
-static int runucs(int argc, char **argv){
+/* parse arguments of cstr command */
+static int runcstr(int argc, char **argv){
   char *path = NULL;
   bool dec = false;
+  bool js = false;
   for(int i = 2; i < argc; i++){
     if(!path && argv[i][0] == '-'){
       if(!strcmp(argv[i], "-d")){
         dec = true;
+      } else if(!strcmp(argv[i], "-js")){
+        js = true;
       } else {
         usage();
       }
@@ -559,7 +577,50 @@ static int runucs(int argc, char **argv){
     eprintf("%s: cannot open", path ? path : "(stdin)");
     return 1;
   }
-  int rv = procucs(ibuf, isiz, dec);
+  int rv = proccstr(ibuf, isiz, dec, js);
+  if(path && path[0] == '@') printf("\n");
+  tcfree(ibuf);
+  return rv;
+}
+
+
+/* parse arguments of ucs command */
+static int runucs(int argc, char **argv){
+  char *path = NULL;
+  bool dec = false;
+  bool un = false;
+  char *kw = NULL;
+  for(int i = 2; i < argc; i++){
+    if(!path && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-d")){
+        dec = true;
+      } else if(!strcmp(argv[i], "-un")){
+        un = true;
+      } else if(!strcmp(argv[i], "-kw")){
+        if(++i >= argc) usage();
+        kw = argv[i];
+      } else {
+        usage();
+      }
+    } else if(!path){
+      path = argv[i];
+    } else {
+      usage();
+    }
+  }
+  char *ibuf;
+  int isiz;
+  if(path && path[0] == '@'){
+    isiz = strlen(path) - 1;
+    ibuf = tcmemdup(path + 1, isiz);
+  } else {
+    ibuf = tcreadfile(path, -1, &isiz);
+  }
+  if(!ibuf){
+    eprintf("%s: cannot open", path ? path : "(stdin)");
+    return 1;
+  }
+  int rv = procucs(ibuf, isiz, dec, un, kw);
   if(path && path[0] == '@') printf("\n");
   tcfree(ibuf);
   return rv;
@@ -605,6 +666,43 @@ static int runhash(int argc, char **argv){
 }
 
 
+/* parse arguments of cipher command */
+static int runcipher(int argc, char **argv){
+  char *path = NULL;
+  char *key = NULL;
+  for(int i = 2; i < argc; i++){
+    if(!path && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-key")){
+        if(++i >= argc) usage();
+        key = argv[i];
+      } else {
+        usage();
+      }
+    } else if(!path){
+      path = argv[i];
+    } else {
+      usage();
+    }
+  }
+  char *ibuf;
+  int isiz;
+  if(path && path[0] == '@'){
+    isiz = strlen(path) - 1;
+    ibuf = tcmemdup(path + 1, isiz);
+  } else {
+    ibuf = tcreadfile(path, -1, &isiz);
+  }
+  if(!ibuf){
+    eprintf("%s: cannot open", path ? path : "(stdin)");
+    return 1;
+  }
+  int rv = proccipher(ibuf, isiz, key);
+  if(path && path[0] == '@') printf("\n");
+  tcfree(ibuf);
+  return rv;
+}
+
+
 /* parse arguments of date command */
 static int rundate(int argc, char **argv){
   char *str = NULL;
@@ -635,6 +733,46 @@ static int rundate(int argc, char **argv){
 }
 
 
+/* parse arguments of tmpl command */
+static int runtmpl(int argc, char **argv){
+  char *path = NULL;
+  TCMAP *vars = tcmpoolmapnew(tcmpoolglobal());
+  for(int i = 2; i < argc; i++){
+    if(!path && argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-var")){
+        if(++i >= argc) usage();
+        const char *name = argv[i];
+        if(++i >= argc) usage();
+        const char *value = argv[i];
+        tcmapput2(vars, name, value);
+      } else {
+        usage();
+      }
+    } else if(!path){
+      path = argv[i];
+    } else {
+      usage();
+    }
+  }
+  char *ibuf;
+  int isiz;
+  if(path && path[0] == '@'){
+    isiz = strlen(path) - 1;
+    ibuf = tcmemdup(path + 1, isiz);
+  } else {
+    ibuf = tcreadfile(path, -1, &isiz);
+  }
+  if(!ibuf){
+    eprintf("%s: cannot open", path ? path : "(stdin)");
+    return 1;
+  }
+  int rv = proctmpl(ibuf, isiz, vars);
+  if(path && path[0] == '@') printf("\n");
+  tcfree(ibuf);
+  return rv;
+}
+
+
 /* parse arguments of conf command */
 static int runconf(int argc, char **argv){
   int mode = 0;
@@ -967,9 +1105,54 @@ static int procxml(const char *ibuf, int isiz, bool dec, bool br){
 }
 
 
+/* perform cstr command */
+static int proccstr(const char *ibuf, int isiz, bool dec, bool js){
+  if(js){
+    if(dec){
+      char *ostr = tcjsonunescape(ibuf);
+      printf("%s", ostr);
+      tcfree(ostr);
+    } else {
+      char *ostr = tcjsonescape(ibuf);
+      printf("%s", ostr);
+      tcfree(ostr);
+    }
+  } else {
+    if(dec){
+      char *ostr = tccstrunescape(ibuf);
+      printf("%s", ostr);
+      tcfree(ostr);
+    } else {
+      char *ostr = tccstrescape(ibuf);
+      printf("%s", ostr);
+      tcfree(ostr);
+    }
+  }
+  return 0;
+}
+
+
 /* perform ucs command */
-static int procucs(const char *ibuf, int isiz, bool dec){
-  if(dec){
+static int procucs(const char *ibuf, int isiz, bool dec, bool un, const char *kw){
+  if(un){
+    uint16_t *ary = tcmalloc(isiz * sizeof(uint16_t) + 1);
+    int anum;
+    tcstrutftoucs(ibuf, ary, &anum);
+    anum = tcstrucsnorm(ary, anum, TCUNSPACE | TCUNLOWER | TCUNNOACC | TCUNWIDTH);
+    char *str = tcmalloc(anum * 3 + 1);
+    tcstrucstoutf(ary, anum, str);
+    printf("%s", str);
+    tcfree(str);
+    tcfree(ary);
+  } else if(kw){
+    TCLIST *words = tcstrtokenize(kw);
+    TCLIST *texts = tcstrkwic(ibuf, words, 10, TCKWMUTAB);
+    for(int i = 0; i < tclistnum(texts); i++){
+      printf("%s\n", tclistval2(texts, i));
+    }
+    tclistdel(texts);
+    tclistdel(words);
+  } else if(dec){
     uint16_t *ary = tcmalloc(isiz + 1);
     int anum = 0;
     for(int i = 0; i < isiz; i += 2){
@@ -1012,6 +1195,22 @@ static int prochash(const char *ibuf, int isiz, bool crc, int ch){
 }
 
 
+/* perform cipher command */
+static int proccipher(const char *ibuf, int isiz, const char *key){
+  char *obuf = tcmalloc(isiz + 1);
+  const char *kbuf = "";
+  int ksiz = 0;
+  if(key){
+    kbuf = key;
+    ksiz = strlen(key);
+  }
+  tcarccipher(ibuf, isiz, kbuf, ksiz, obuf);
+  fwrite(obuf, 1, isiz, stdout);
+  tcfree(obuf);
+  return 0;
+}
+
+
 /* perform date command */
 static int procdate(const char *str, int jl, bool wf, bool rf){
   int64_t t = str ? tcstrmktime(str) : time(NULL);
@@ -1030,6 +1229,18 @@ static int procdate(const char *str, int jl, bool wf, bool rf){
 }
 
 
+/* perform tmpl command */
+static int proctmpl(const char *ibuf, int isiz, TCMAP *vars){
+  TCTMPL *tmpl = tctmplnew();
+  tctmplload(tmpl, ibuf);
+  char *str = tctmpldump(tmpl, vars);
+  printf("%s", str);
+  tcfree(str);
+  tctmpldel(tmpl);
+  return 0;
+}
+
+
 /* perform conf command */
 static int procconf(int mode){
   switch(mode){
@@ -1090,6 +1301,20 @@ static int procconf(int mode){
     printf("sysconf(_SC_CLK_TCK): %ld\n", sysconf(_SC_CLK_TCK));
     printf("sysconf(_SC_OPEN_MAX): %ld\n", sysconf(_SC_OPEN_MAX));
     printf("sysconf(_SC_PAGESIZE): %ld\n", sysconf(_SC_PAGESIZE));
+    TCMAP *info = tcsysinfo();
+    if(info){
+      const char *val = tcmapget2(info, "size");
+      if(val) printf("sysinfo(size): %s\n", val);
+      val = tcmapget2(info, "rss");
+      if(val) printf("sysinfo(rss): %s\n", val);
+      val = tcmapget2(info, "total");
+      if(val) printf("sysinfo(total): %s\n", val);
+      val = tcmapget2(info, "free");
+      if(val) printf("sysinfo(free): %s\n", val);
+      val = tcmapget2(info, "cached");
+      if(val) printf("sysinfo(cached): %s\n", val);
+      tcmapdel(info);
+    }
     struct stat sbuf;
     if(stat(MYCDIRSTR, &sbuf) == 0){
       printf("stat(st_uid): %d\n", (int)sbuf.st_uid);
