@@ -81,7 +81,7 @@ int main(int argc, char **argv){
     usage();
   }
   if(rv != 0){
-    printf("FAILED:");
+    printf("FAILED: TCRNDSEED=%u PID=%d", g_randseed, (int)getpid());
     for(int i = 0; i < argc; i++){
       printf(" %s", argv[i]);
     }
@@ -880,11 +880,24 @@ static int procmisc(int rnum){
           if(dary[j] != dary[j]) err = true;
         }
       }
+      tcstrutfnorm(ustr, TCUNSPACE | TCUNLOWER | TCUNNOACC | TCUNWIDTH);
+      if(tcstrucsnorm(dary, danum, TCUNSPACE | TCUNLOWER | TCUNNOACC | TCUNWIDTH) > danum)
+        err = true;
+      list = tcstrtokenize("a ab abc b bc bcd abcde \"I'm nancy\" x \"xx");
+      if(tclistnum(list) != 10) err = true;
+      int opts = myrand(TCKWMUBRCT + 1);
+      if(myrand(2) == 0) opts |= TCKWNOOVER;
+      if(myrand(2) == 0) opts |= TCKWPULEAD;
+      TCLIST *texts = tcstrkwic(ustr, list, myrand(10), opts);
+      tclistdel(texts);
+      tclistdel(list);
       list = tclistnew3("hop", "step", "jump", "touchdown", NULL);
       if(tclistnum(list) != 4) err = true;
+      tclistprintf(list, "%s:%010d:%7.3f", "game set", 123456789, 12345.6789);
       tclistdel(list);
       map = tcmapnew3("hop", "step", "jump", "touchdown", NULL);
       if(tcmaprnum(map) != 2) err = true;
+      tcmapprintf(map, "joker", "%s:%010d:%7.3f", "game set", 123456789, 12345.6789);
       tcmapdel(map);
       list = tcstrsplit(",a,b..c,d,", ",.");
       if(tclistnum(list) != 7) err = true;
@@ -948,6 +961,13 @@ static int procmisc(int rnum){
         tcchidxhash(chidx, kbuf, ksiz);
       }
       tcchidxdel(chidx);
+      char kbuf[TCNUMBUFSIZ];
+      int ksiz = sprintf(kbuf, "%d", myrand(200));
+      char *enc = tcmalloc(slen + 1);
+      tcarccipher(str, slen, kbuf, ksiz, enc);
+      tcarccipher(enc, slen, kbuf, ksiz, enc);
+      if(memcmp(enc, str, slen)) err = true;
+      tcfree(enc);
       buf = tczeromap(myrand(1024*256) + 1);
       tczerounmap(buf);
     }
@@ -986,6 +1006,14 @@ static int procmisc(int rnum){
         vbuf = tctreeget2(tree, kbuf);
         if(*(double *)vbuf < 1.0) err = true;
       }
+      for(int j = 0; j < 10; j++){
+        char kbuf[RECBUFSIZ];
+        sprintf(kbuf, "%d", myrand(10));
+        tcmapprintf(map, kbuf, "%s:%f", kbuf, (double)i * j);
+        tctreeprintf(tree, kbuf, "%s:%f", kbuf, (double)i * j);
+      }
+      if(!tcmapget4(map, "ace", "dummy")) err = true;
+      if(!tctreeget4(tree, "ace", "dummy")) err = true;
       tctreedel(tree);
       tcmapdel(map);
     }
@@ -1153,7 +1181,28 @@ static int procmisc(int rnum){
     if(strcmp(dec, str)) err = true;
     tcfree(dec);
     tcfree(buf);
+    buf = tccstrescape(str);
+    if(strcmp(buf, "5%2+3-1=4 \\x22Yes/No\\x22 <a&b>")) err = true;
+    dec = tccstrunescape(buf);
+    if(strcmp(dec, str)) err = true;
+    tcfree(dec);
+    tcfree(buf);
+    buf = tcjsonescape(str);
+    if(strcmp(buf, "5%2+3-1=4 \\u0022Yes/No\\u0022 <a&b>")) err = true;
+    dec = tcjsonunescape(buf);
+    if(strcmp(dec, str)) err = true;
+    tcfree(dec);
+    tcfree(buf);
     if(i % 10 == 1){
+      TCMAP *params = tcmapnew3("one", "=first=", "two", "&second&", "three", "#third#", NULL);
+      char *estr = tcwwwformencode(params);
+      TCMAP *nparams = tcmapnew2(1);
+      tcwwwformdecode(estr, nparams);
+      if(strcmp(estr, "one=%3Dfirst%3D&two=%26second%26&three=%23third%23") ||
+         tcmaprnum(nparams) != tcmaprnum(params) || !tcmapget2(nparams, "two")) err = true;
+      tcmapdel(nparams);
+      tcfree(estr);
+      tcmapdel(params);
       list = tcxmlbreak("<abc de=\"foo&amp;\" gh='&lt;bar&gt;'>xyz<br>\na<!--<mikio>--></abc>");
       for(int j = 0; j < tclistnum(list); j++){
         const char *elem = tclistval2(list, j);
@@ -1162,6 +1211,53 @@ static int procmisc(int rnum){
       }
       tclistdel(list);
     }
+    if(i % 100 == 1){
+      TCTMPL *tmpl = tctmplnew();
+      const char *str =
+        "{{ title XML }}{{UNKNOWN COMMAND}}{{CONF author 'Mikio Hirabayashi'}}\n"
+        "{{ FOREACH docs \\}}\n"
+        "{{ IF void }}===={{void}}{{ ELSE }}----{{void.void}}{{ END }}\n"
+        "{{ IF .id }}ID:{{ .id }}{{ END }}\n"
+        "{{ IF .title }}Title:{{ .title }}{{ END }}\n"
+        "{{ IF author }}Author:{{ author }}{{ END }}\n"
+        "{{ IF .coms }}{{ SET addr 'Setagaya, Tokyo' }}--\n"
+        "{{ FOREACH .coms com }}{{ com.author MD5 }}: {{ com.body }}\n"
+        "{{ END \\}}\n"
+        "{{ END \\}}\n"
+        "{{ END \\}}\n";
+      tctmplsetsep(tmpl, "{{", "}}");
+      tctmplload(tmpl, str);
+      const char *cval = tctmplconf(tmpl, "author");
+      if(!cval || strcmp(cval, "Mikio Hirabayashi")) err = true;
+      TCMPOOL *mpool = tcmpoolnew();
+      TCMAP *vars = tcmpoolmapnew(mpool);
+      tcmapput2(vars, "title", "I LOVE YOU");
+      TCLIST *docs = tcmpoollistnew(mpool);
+      for(int j = 0; j < 3; j++){
+        TCMAP *doc = tcmpoolmapnew(mpool);
+        char vbuf[TCNUMBUFSIZ];
+        sprintf(vbuf, "%d", i + j);
+        tcmapput2(doc, "id", vbuf);
+        sprintf(vbuf, "[%08d]", i + j);
+        tcmapput2(doc, "title", vbuf);
+        TCLIST *coms = tcmpoollistnew(mpool);
+        for(int k = 0; k < 3; k++){
+          TCMAP *com = tcmpoolmapnew(mpool);
+          sprintf(vbuf, "u%d", k);
+          tcmapput2(com, "author", vbuf);
+          sprintf(vbuf, "this is the %dth pen.", (j + 1) * (k + 1) + i);
+          tcmapput2(com, "body", vbuf);
+          tclistpushmap(coms, com);
+        }
+        tcmapputlist(doc, "coms", coms);
+        tclistpushmap(docs, doc);
+      }
+      tcmapputlist(vars, "docs", docs);
+      char *res = tctmpldump(tmpl, vars);
+      tcfree(res);
+      tcmpooldel(mpool);
+      tctmpldel(tmpl);
+    }
     if(i % 10 == 1){
       for(int16_t j = 1; j <= 0x2000; j *= 2){
         for(int16_t num = j - 1; num <= j + 1; num++){
@@ -1191,7 +1287,7 @@ static int procmisc(int rnum){
           if(num != nnum || step != nstep) err = true;
         }
       }
-      char *bitmap = TCBITMAPNEW(100);
+      TCBITMAP *bitmap = TCBITMAPNEW(100);
       for(int j = 0; j < 100; j++){
         if(j % 3 == 0) TCBITMAPON(bitmap, j);
         if(j % 5 == 0) TCBITMAPOFF(bitmap, j);
@@ -1222,6 +1318,15 @@ static int procmisc(int rnum){
       }
       tcfree(buf);
     }
+    if(i % 100 == 1){
+      char path[RECBUFSIZ];
+      sprintf(path, "%d", myrand(10));
+      if(tcpathlock(path)){
+        if(!tcpathunlock(path)) err = true;
+      } else {
+        err = true;
+      }
+    }
     if(rnum > 250 && i % (rnum / 250) == 0){
       iputchar('.');
       if(i == rnum || i % (rnum / 10) == 0) iprintf(" (%08d)\n", i);
@@ -1243,19 +1348,19 @@ static int procwicked(int rnum){
   double stime = tctime();
   TCMPOOL *mpool = tcmpoolglobal();
   TCXSTR *xstr = myrand(2) > 0 ? tcxstrnew() : tcxstrnew2("hello world");
-  tcmpoolputxstr(mpool, xstr);
+  tcmpoolpushxstr(mpool, xstr);
   TCLIST *list = myrand(2) > 0 ? tclistnew() : tclistnew2(myrand(rnum) + rnum / 2);
-  tcmpoolputlist(mpool, list);
+  tcmpoolpushlist(mpool, list);
   TCPTRLIST *ptrlist = myrand(2) > 0 ? tcptrlistnew() : tcptrlistnew2(myrand(rnum) + rnum / 2);
-  tcmpoolput(mpool, ptrlist, (void (*)(void*))tcptrlistdel);
+  tcmpoolpush(mpool, ptrlist, (void (*)(void*))tcptrlistdel);
   TCMAP *map = myrand(2) > 0 ? tcmapnew() : tcmapnew2(myrand(rnum) + rnum / 2);
-  tcmpoolputmap(mpool, map);
+  tcmpoolpushmap(mpool, map);
   TCTREE *tree = myrand(2) > 0 ? tctreenew() : tctreenew2(tccmpdecimal, NULL);
-  tcmpoolputtree(mpool, tree);
+  tcmpoolpushtree(mpool, tree);
   TCMDB *mdb = myrand(2) > 0 ? tcmdbnew() : tcmdbnew2(myrand(rnum) + rnum / 2);
-  tcmpoolput(mpool, mdb, (void (*)(void*))tcmdbdel);
+  tcmpoolpush(mpool, mdb, (void (*)(void*))tcmdbdel);
   TCNDB *ndb = myrand(2) > 0 ? tcndbnew() : tcndbnew2(tccmpdecimal, NULL);
-  tcmpoolput(mpool, ndb, (void (*)(void*))tcndbdel);
+  tcmpoolpush(mpool, ndb, (void (*)(void*))tcndbdel);
   for(int i = 1; i <= rnum; i++){
     char kbuf[RECBUFSIZ];
     int ksiz = sprintf(kbuf, "%d", myrand(i));
@@ -1672,23 +1777,78 @@ static int procwicked(int rnum){
       break;
     case 63:
       iputchar('+');
-      if(myrand(100) == 0) tcmpoolmalloc(mpool, 1);
+      if(myrand(100) == 0){
+        char *tptr = tcmpoolmalloc(mpool, 1);
+        switch(myrand(5)){
+        case 0:
+          tcfree(tptr);
+          tcmpoolpop(mpool, false);
+          break;
+        case 1:
+          tcmpoolpop(mpool, true);
+          break;
+        }
+      }
       break;
     case 64:
       iputchar('+');
-      if(myrand(100) == 0) tcmpoolxstrnew(mpool);
+      if(myrand(100) == 0){
+        TCXSTR *txstr = tcmpoolxstrnew(mpool);
+        switch(myrand(5)){
+        case 0:
+          tcxstrdel(txstr);
+          tcmpoolpop(mpool, false);
+          break;
+        case 1:
+          tcmpoolpop(mpool, true);
+          break;
+        }
+      }
       break;
     case 65:
       iputchar('+');
-      if(myrand(100) == 0) tcmpoollistnew(mpool);
+      if(myrand(100) == 0){
+        TCLIST *tlist = tcmpoollistnew(mpool);
+        switch(myrand(5)){
+        case 0:
+          tclistdel(tlist);
+          tcmpoolpop(mpool, false);
+          break;
+        case 1:
+          tcmpoolpop(mpool, true);
+          break;
+        }
+      }
       break;
     case 66:
       iputchar('+');
-      if(myrand(100) == 0) tcmpoolmapnew(mpool);
+      if(myrand(100) == 0){
+        TCMAP *tmap = tcmpoolmapnew(mpool);
+        switch(myrand(5)){
+        case 0:
+          tcmapdel(tmap);
+          tcmpoolpop(mpool, false);
+          break;
+        case 1:
+          tcmpoolpop(mpool, true);
+          break;
+        }
+      }
       break;
     case 67:
       iputchar('+');
-      if(myrand(100) == 0) tcmpooltreenew(mpool);
+      if(myrand(100) == 0){
+        TCTREE *ttree = tcmpooltreenew(mpool);
+        switch(myrand(5)){
+        case 0:
+          tctreedel(ttree);
+          tcmpoolpop(mpool, false);
+          break;
+        case 1:
+          tcmpoolpop(mpool, true);
+          break;
+        }
+      }
       break;
     default:
       iputchar('@');
